/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif

#include "dsmapi-procedures.h"

#include "ds-assistant.h"
#include "hwdesc_attributes.h"

namespace dSA
{

enum
{
    b4_cp_load1_active = 15,
    b4_cp_load2_active = 16,
    b4_cp_load1_standby = 17,
    b4_cp_load2_standby = 18,
};

/* throws exception */
void setChargingPulse(const Device &dev, unsigned cp_id)
{
    dS_Assistant *dsa = dS_Assistant::getInstance();
    boost::shared_ptr<dSMApiWrapper> dsmapi = dsa->dS485();

    assert(cp_id < charge_slot_count);
    cp_parameters_t cp_parms = charging_pulse_table[cp_id];

    // writing 2 values is not atomic
    // but thhe values in the table are redundant,
    // - LOAD1 has only 2 possible values
    // - LOAD2 each value exists for both LOAD1 values, except 0x76 and 0x86
    // TODO based on current slot, shuffle writes in a smart way

    dsmapi->setConfigurationValue(dev.getDSMdSUID(), dev.getId(), 4,
                                  b4_cp_load1_active, cp_parms.LOAD1);
    dsmapi->setConfigurationValue(dev.getDSMdSUID(), dev.getId(), 4,
                                  b4_cp_load1_standby, cp_parms.LOAD1);


    dsmapi->setConfigurationValue(dev.getDSMdSUID(), dev.getId(),
                                  4, b4_cp_load2_active,
                                  cp_parms.LOAD2);
    dsmapi->setConfigurationValue(dev.getDSMdSUID(), dev.getId(), 4,
                                  b4_cp_load2_standby, cp_parms.LOAD2);
}

void resetChargingPulse(const_device_p dev)
{
    dS_Assistant *dsa = dS_Assistant::getInstance();
    boost::shared_ptr<dSMApiWrapper> dsmapi = dsa->dS485();

    dsmapi->setConfigurationValue(dev->getDSMdSUID(), dev->getId(), 4,
                                  b4_cp_load1_active, 0x6c);
    dsmapi->setConfigurationValue(dev->getDSMdSUID(), dev->getId(), 4,
                                  b4_cp_load2_active, 0x8a);


    dsmapi->setConfigurationValue(dev->getDSMdSUID(), dev->getId(),
                                  4, b4_cp_load1_standby, 0x3c);
    dsmapi->setConfigurationValue(dev->getDSMdSUID(), dev->getId(), 4,
                                  b4_cp_load2_standby, 0x94);
}

} // namespace

